@@ -1,22 +1,22 @@
 /* mpfr_acosh -- inverse hyperbolic cosine
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -73,7 +73,7 @@ mpfr_acosh (mpfr_ptr y, mpfr_srcptr x , mp_rnd_t rnd_mode)
     /* Declaration of the size variables */
     mp_prec_t Ny = MPFR_PREC(y);   /* Precision of output variable */
     mp_prec_t Nt;                  /* Precision of the intermediary variable */
-    mp_exp_t  err, exp_te, exp_ti; /* Precision of error */
+    mp_exp_t  err, exp_te, d;      /* Precision of error */
     MPFR_ZIV_DECL (loop);
 
     /* compute the precision of intermediary variable */
@@ -87,19 +87,56 @@ mpfr_acosh (mpfr_ptr y, mpfr_srcptr x , mp_rnd_t rnd_mode)
     MPFR_ZIV_INIT (loop, Nt);
     for (;;)
       {
+        MPFR_BLOCK_DECL (flags);
+
         /* compute acosh */
-        mpfr_mul (t, x, x, GMP_RNDD);      /* x^2 */
-        exp_te = MPFR_GET_EXP (t);
-        mpfr_sub_ui (t, t, 1, GMP_RNDD);   /* x^2-1 */
-        exp_ti = MPFR_GET_EXP (t);
-        mpfr_sqrt (t, t, GMP_RNDN);        /* sqrt(x^2-1) */
-        mpfr_add (t, t, x, GMP_RNDN);      /* sqrt(x^2-1)+x */
-        mpfr_log (t, t, GMP_RNDN);         /* ln(sqrt(x^2-1)+x)*/
+        MPFR_BLOCK (flags, mpfr_mul (t, x, x, GMP_RNDD));  /* x^2 */
+        if (MPFR_OVERFLOW (flags))
+          {
+            mpfr_t ln2;
+            mp_prec_t pln2;
+
+            /* As x is very large and the precision is not too large, we
+               assume that we obtain the same result by evaluating ln(2x).
+               We need to compute ln(x) + ln(2) as 2x can overflow. TODO:
+               write a proof and add an MPFR_ASSERTN. */
+            mpfr_log (t, x, GMP_RNDN);  /* err(log) < 1/2 ulp(t) */
+            pln2 = Nt - MPFR_PREC_MIN < MPFR_GET_EXP (t) ?
+              MPFR_PREC_MIN : Nt - MPFR_GET_EXP (t);
+            mpfr_init2 (ln2, pln2);
+            mpfr_const_log2 (ln2, GMP_RNDN);  /* err(ln2) < 1/2 ulp(t) */
+            mpfr_add (t, t, ln2, GMP_RNDN);  /* err <= 3/2 ulp(t) */
+            mpfr_clear (ln2);
+            err = 1;
+          }
+        else
+          {
+            exp_te = MPFR_GET_EXP (t);
+            mpfr_sub_ui (t, t, 1, GMP_RNDD);   /* x^2-1 */
+            if (MPFR_UNLIKELY (MPFR_IS_ZERO (t)))
+              {
+                /* This means that x is very close to 1: x = 1 + t with
+                   t < 2^(-Nt). We have: acosh(x) = sqrt(2t) (1 - eps(t))
+                   with 0 < eps(t) < t / 12. */
+                mpfr_sub_ui (t, x, 1, GMP_RNDD);   /* t = x - 1 */
+                mpfr_mul_2ui (t, t, 1, GMP_RNDN);  /* 2t */
+                mpfr_sqrt (t, t, GMP_RNDN);        /* sqrt(2t) */
+                err = 1;
+              }
+            else
+              {
+                d = exp_te - MPFR_GET_EXP (t);
+                mpfr_sqrt (t, t, GMP_RNDN);        /* sqrt(x^2-1) */
+                mpfr_add (t, t, x, GMP_RNDN);      /* sqrt(x^2-1)+x */
+                mpfr_log (t, t, GMP_RNDN);         /* ln(sqrt(x^2-1)+x) */
+
+                /* error estimate -- see algorithms.tex */
+                err = 3 + MAX (1, d) - MPFR_GET_EXP (t);
+                /* error is bounded by 1/2 + 2^err <= 2^(max(0,1+err)) */
+                err = MAX (0, 1 + err);
+              }
+          }
 
-        /* error estimate -- see algorithms.tex */
-        err = 3 + MAX (1, exp_te - exp_ti) - MPFR_GET_EXP(t);
-        /* error is bounded by 1/2 + 2^err <= 2^(1+max(-1,err)) */
-        err = 1 + MAX (-1, err);
         if (MPFR_LIKELY (MPFR_CAN_ROUND (t, Nt - err, Ny, rnd_mode)))
           break;
 
@@ -117,9 +154,3 @@ mpfr_acosh (mpfr_ptr y, mpfr_srcptr x , mp_rnd_t rnd_mode)
   MPFR_SAVE_EXPO_FREE (expo);
   return mpfr_check_range (y, inexact, rnd_mode);
 }
-
-
-
-
-
-
