@@ -1,23 +1,23 @@
 /* mpfr_get_str -- output a floating-point number to a string
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 Contributed by Alain Delplanque and Paul Zimmermann.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -29,6 +29,8 @@ MA 02110-1301, USA. */
 static int mpfr_get_str_aux (char *const, mp_exp_t *const, mp_limb_t *const,
                        mp_size_t, mp_exp_t, long, int, size_t, mp_rnd_t);
 
+/* The implicit \0 is useless, but we do not write num_to_text[36]
+   otherwise g++ complains. */
 static const char num_to_text[] = "0123456789abcdefghijklmnopqrstuvwxyz";
 
 /* copy most important limbs of {op, n2} in {rp, n1} */
@@ -1403,7 +1405,7 @@ mpfr_get_str (char *s, mp_exp_t *e, int b, size_t m, mpfr_srcptr x, mp_rnd_t rnd
   mp_limb_t *a;
   mp_exp_t exp_a;
   mp_limb_t *result;
-  mp_limb_t *xp, *x1;
+  mp_limb_t *xp;
   mp_limb_t *reste;
   size_t nx, nx1;
   size_t n, i;
@@ -1590,6 +1592,8 @@ mpfr_get_str (char *s, mp_exp_t *e, int b, size_t m, mpfr_srcptr x, mp_rnd_t rnd
         }
       else if ((mp_exp_t) m > g) /* we have to multiply x by b^exp */
         {
+          mp_limb_t *x1;
+
           /* a2*2^exp_a =  b^e */
           err = mpfr_mpn_exp (a, &exp_a, b, exp, n);
           /* here, the error on a is at most 2^err ulps */
@@ -1627,6 +1631,8 @@ mpfr_get_str (char *s, mp_exp_t *e, int b, size_t m, mpfr_srcptr x, mp_rnd_t rnd
         }
       else
         {
+          mp_limb_t *x1;
+
           /* a2*2^exp_a =  b^e */
           err = mpfr_mpn_exp (a, &exp_a, b, exp, n);
           exact = (err == -1);
