@@ -1,22 +1,22 @@
 /* Utilities for MPFR developers, not exported.
 
-Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -31,7 +31,11 @@ MA 02110-1301, USA. */
    (which is very unlikely and probably means something broken in
    this source file), we should do that with some macro (that would
    also force to disable incompatible features). */
+#if defined (__cplusplus)
+#include <cstdio>
+#else
 #include <stdio.h>
+#endif
 #include <limits.h>
 
 /* Check if we are inside a build of MPFR or inside the test suite.
@@ -47,10 +51,8 @@ MA 02110-1301, USA. */
 
 /* Include 'config.h' before using ANY configure macros if needed
    NOTE: It isn't MPFR 'config.h', but GMP's one! */
-#if defined(HAVE_CONFIG_H)
-#if HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ifdef HAVE_CONFIG_H
+# include "config.h"
 #endif
 
 #ifdef  MPFR_HAVE_GMP_IMPL /* Build with gmp internals*/
@@ -86,6 +88,13 @@ MA 02110-1301, USA. */
 #endif
 #undef MPFR_NEED_LONGLONG_H
 
+/* For the definition of MPFR_THREAD_ATTR. GCC/ICC detection macros are
+   no longer used, as they sometimes gave incorrect information about
+   the support of thread-local variables. A configure check is now done.
+   If the use of detection macros is needed in the future, this could be
+   moved below (after the detection macros are defined). */
+#include "mpfr-thread.h"
+
 
 /******************************************************
  ***************** Detection macros *******************
@@ -100,31 +109,35 @@ MA 02110-1301, USA. */
 # define __MPFR_STDC(version) 0
 #endif
 
-#if defined(__GNUC__) && defined(__GNUC_MINOR__) && !defined(__ICC)
-# define __MPFR_GNUC(a, i) \
- (MPFR_VERSION_NUM(__GNUC__,__GNUC_MINOR__,0)>=MPFR_VERSION_NUM(a,i,0))
+#if defined(__ICC)
+# define __MPFR_ICC(a,b,c) (__ICC >= (a)*100+(b)*10+(c))
+#elif defined(__INTEL_COMPILER)
+# define __MPFR_ICC(a,b,c) (__INTEL_COMPILER >= (a)*100+(b)*10+(c))
 #else
-# define __MPFR_GNUC(a, i) 0
+# define __MPFR_ICC(a,b,c) 0
+#endif
+
+#if defined(__GNUC__) && defined(__GNUC_MINOR__) && ! __MPFR_ICC(0,0,0)
+# define __MPFR_GNUC(a,i) \
+ (MPFR_VERSION_NUM(__GNUC__,__GNUC_MINOR__,0) >= MPFR_VERSION_NUM(a,i,0))
+#else
+# define __MPFR_GNUC(a,i) 0
 #endif
 
 #if defined(__GLIBC__) && defined(__GLIBC_MINOR__)
-# define __MPFR_GLIBC(a, i) \
- (MPFR_VERSION_NUM(__GLIBC__,__GLIBC_MINOR__,0)>=MPFR_VERSION_NUM(a,i,0))
+# define __MPFR_GLIBC(a,i) \
+ (MPFR_VERSION_NUM(__GLIBC__,__GLIBC_MINOR__,0) >= MPFR_VERSION_NUM(a,i,0))
 #else
-# define __MPFR_GLIBC(a, i) 0
+# define __MPFR_GLIBC(a,i) 0
 #endif
 
-#if defined(__GNU_MP_VERSION)&&defined(__GNU_MP_VERSION_MINOR)&&defined(__GNU_MP_VERSION_PATCHLEVEL)
-# define __MPFR_GMP(a, b, c) \
-(MPFR_VERSION_NUM(__GNU_MP_VERSION,__GNU_MP_VERSION_MINOR,__GNU_MP_VERSION_PATCHLEVEL) >= MPFR_VERSION_NUM(a,b,c))
+#if defined(__GNU_MP_VERSION) && \
+    defined(__GNU_MP_VERSION_MINOR) && \
+    defined(__GNU_MP_VERSION_PATCHLEVEL)
+# define __MPFR_GMP(a,b,c) \
+  (MPFR_VERSION_NUM(__GNU_MP_VERSION,__GNU_MP_VERSION_MINOR,__GNU_MP_VERSION_PATCHLEVEL) >= MPFR_VERSION_NUM(a,b,c))
 #else
-# define __MPFR_GMP(a, b, c) 0
-#endif
-
-#if defined(__ICC)
-# define __MPFR_ICC(a,b,c) (__ICC >= (a)*100+(b)*10+c)
-#else
-# define __MPFR_ICC(a,b,c) 0
+# define __MPFR_GMP(a,b,c) 0
 #endif
 
 
@@ -171,29 +184,6 @@ MA 02110-1301, USA. */
  ************* Global Internal Variables **************
  ******************************************************/
 
-/* Note: Let's define MPFR_THREAD_ATTR even after a #error to make the
-   error message more visible (e.g. gcc doesn't immediately stop after
-   the #error line and outputs many error messages if MPFR_THREAD_ATTR
-   is not defined). But some compilers will just output a message and
-   may build MPFR "successfully" (without thread support). */
-#ifdef MPFR_USE_THREAD_SAFE
-# if __MPFR_GNUC(3,3) || __MPFR_ICC(8,1,0)
-#  define MPFR_THREAD_ATTR __thread
-# elif defined(_MSC_VER)
-#  if defined(_WINDLL)
-#   error "Can't build MPFR DLL as thread safe."
-#   define MPFR_THREAD_ATTR
-#  else
-#   define MPFR_THREAD_ATTR __declspec( thread )
-#  endif
-# else
-#  error "Can't build MPFR as thread safe (you need gcc >= 3.3 or icc >= 8.1)."
-#  define MPFR_THREAD_ATTR
-# endif
-#else
-# define MPFR_THREAD_ATTR
-#endif
-
 /* Cache struct */
 struct __gmpfr_cache_s {
   mpfr_t x;
@@ -219,6 +209,10 @@ __MPFR_DECLSPEC extern MPFR_THREAD_ATTR mpfr_cache_t __gmpfr_cache_const_catalan
 #define BASE_MAX 36
 __MPFR_DECLSPEC extern const __mpfr_struct __gmpfr_l2b[BASE_MAX-1][2];
 
+/* Note: do not use the following values when they can be outside the
+   current exponent range, e.g. when the exponent range has not been
+   extended yet; under such a condition, they can be used only in
+   mpfr_cmpabs. */
 __MPFR_DECLSPEC extern const mpfr_t __gmpfr_one;
 __MPFR_DECLSPEC extern const mpfr_t __gmpfr_two;
 __MPFR_DECLSPEC extern const mpfr_t __gmpfr_four;
@@ -265,6 +259,40 @@ __MPFR_DECLSPEC extern const mpfr_t __gmpfr_four;
 #define mpfr_erangeflag_p() \
   ((int) (__gmpfr_flags & MPFR_FLAGS_ERANGE))
 
+/* Testing an exception flag correctly is tricky. There are mainly two
+   pitfalls: First, one needs to remember to clear the corresponding
+   flag, in case it was set before the function call or during some
+   intermediate computations (in practice, one can clear all the flags).
+   Secondly, one needs to test the flag early enough, i.e. before it
+   can be modified by another function. Moreover, it is quite difficult
+   (if not impossible) to reliably check problems with "make check". To
+   avoid these pitfalls, it is recommended to use the following macros.
+   Other use of the exception-flag predicate functions/macros will be
+   detected by mpfrlint.
+   Note: _op can be either a statement or an expression.
+   MPFR_BLOCK_EXCEP should be used only inside a block; it is useful to
+   detect some exception in order to exit the block as soon as possible. */
+#define MPFR_BLOCK_DECL(_flags) unsigned int _flags
+#define MPFR_BLOCK(_flags,_op)          \
+  do                                    \
+    {                                   \
+      mpfr_clear_flags ();              \
+      _op;                              \
+      (_flags) = __gmpfr_flags;         \
+    }                                   \
+  while (0)
+#define MPFR_BLOCK_TEST(_flags,_f) MPFR_UNLIKELY ((_flags) & (_f))
+#define MPFR_BLOCK_EXCEP (__gmpfr_flags & (MPFR_FLAGS_UNDERFLOW | \
+                                           MPFR_FLAGS_OVERFLOW | \
+                                           MPFR_FLAGS_NAN))
+/* Let's use a MPFR_ prefix, because e.g. OVERFLOW is defined by glibc's
+   math.h, though this is not a reserved identifier! */
+#define MPFR_UNDERFLOW(_flags)  MPFR_BLOCK_TEST (_flags, MPFR_FLAGS_UNDERFLOW)
+#define MPFR_OVERFLOW(_flags)   MPFR_BLOCK_TEST (_flags, MPFR_FLAGS_OVERFLOW)
+#define MPFR_NANFLAG(_flags)    MPFR_BLOCK_TEST (_flags, MPFR_FLAGS_NAN)
+#define MPFR_INEXFLAG(_flags)   MPFR_BLOCK_TEST (_flags, MPFR_FLAGS_INEXACT)
+#define MPFR_ERANGEFLAG(_flags) MPFR_BLOCK_TEST (_flags, MPFR_FLAGS_ERANGE)
+
 
 /******************************************************
  ******************** Assertions **********************
@@ -288,16 +316,6 @@ __MPFR_DECLSPEC extern const mpfr_t __gmpfr_four;
 # define MPFR_ASSERTD(expr)  ((void) 0)
 #endif
 
-/* Check if the args are correct  */
-/* Can't be used since TMP variables are not correct */
-#define MPFR_CHECK1(x,r) \
- MPFR_ASSERTD(mpfr_check(x) && GMP_RNDN <= r && r <= GMP_RNDD)
-#define MPFR_CHECK2(x,y,r) \
- MPFR_ASSERTD(mpfr_check(x) && mpfr_check(y) && GMP_RNDN <= r && r <= GMP_RNDD)
-#define MPFR_CHECK3(x,y,z,r) \
- MPFR_ASSERTD(mpfr_check(x) && mpfr_check(y) && mpfr_check(z) && \
-  GMP_RNDN <= r && r <= GMP_RNDD)
-
 /* Code to deal with impossible
    WARNING: It doesn't use do { } while (0) for Insure++*/
 #define MPFR_RET_NEVER_GO_HERE()  {MPFR_ASSERTN(0); return 0;}
@@ -377,7 +395,7 @@ static double double_zero = 0.0;
 # define DBL_NEG_ZERO (-0.0)
 #endif
 
-/* for x of type ieee_double_extract */
+/* Note: the argument x must be a lvalue of type double. */
 #if _GMP_IEEE_FLOATS
 typedef union ieee_double_extract Ieee_double_extract;
 
@@ -389,13 +407,21 @@ typedef union ieee_double_extract Ieee_double_extract;
                          ((((Ieee_double_extract *)&(x))->s.manl != 0) || \
                          (((Ieee_double_extract *)&(x))->s.manh != 0)))
 #else
-# define DOUBLE_ISINF(x) ((x) > DBL_MAX || (x) < -DBL_MAX)
-# if MPFR_NANISNAN
-/* Avoid MIPSpro / IRIX64 (incorrect) optimizations.
-   The + must not be replaced by a ||. */
-#  define DOUBLE_ISNAN(x) (!(((x) >= 0.0) + ((x) <= 0.0)))
+/* Below, the &(x) == &(x) || &(x) != &(x) allows to make sure that x
+   is a lvalue without (probably) any warning from the compiler.  The
+   &(x) != &(x) is needed to avoid a failure under Mac OS X 10.4.11
+   (with Xcode 2.4.1, i.e. the latest one). */
+# define LVALUE(x) (&(x) == &(x) || &(x) != &(x))
+# define DOUBLE_ISINF(x) (LVALUE(x) && ((x) > DBL_MAX || (x) < -DBL_MAX))
+# ifdef MPFR_NANISNAN
+/* Avoid MIPSpro / IRIX64 / gcc -ffast-math (incorrect) optimizations.
+   The + must not be replaced by a ||. With gcc -ffast-math, NaN is
+   regarded as a positive number or something like that; the second
+   test catches this case. */
+#  define DOUBLE_ISNAN(x) \
+    (LVALUE(x) && !((((x) >= 0.0) + ((x) <= 0.0)) && -(x)*(x) <= 0.0))
 # else
-#  define DOUBLE_ISNAN(x) ((x) != (x))
+#  define DOUBLE_ISNAN(x) (LVALUE(x) && (x) != (x))
 # endif
 #endif
 
@@ -416,7 +442,7 @@ typedef union ieee_double_extract Ieee_double_extract;
    Otherwise (e.g. in base 10), we get an upper bound of the
    precision, and correct rounding isn't currently provided.
 */
-#if LDBL_MANT_DIG && FLT_RADIX == 2
+#if defined(LDBL_MANT_DIG) && FLT_RADIX == 2
 # define MPFR_LDBL_MANT_DIG LDBL_MANT_DIG
 #else
 # define MPFR_LDBL_MANT_DIG \
@@ -503,7 +529,7 @@ typedef union {
  *************** _Decimal64 support *******************
  ******************************************************/
 
-#if MPFR_WANT_DECIMAL_FLOATS
+#ifdef MPFR_WANT_DECIMAL_FLOATS
 /* to cast between binary64 and decimal64 */
 union ieee_double_decimal64 { double d; _Decimal64 d64; };
 #endif
@@ -549,8 +575,18 @@ typedef unsigned long int  mpfr_uexp_t;
 #ifndef mp_exp_unsigned_t
 # define mp_exp_unsigned_t mpfr_uexp_t
 #endif
-#define mpfr_get_exp_t(x,r) mpfr_get_si((x),(r))
-#define mpfr_set_exp_t(x,e,r) mpfr_set_si((x),(e),(r))
+
+#if MPFR_EXP_MIN >= LONG_MIN && MPFR_EXP_MAX <= LONG_MAX
+typedef long int mpfr_eexp_t;
+# define mpfr_get_exp_t(x,r) mpfr_get_si((x),(r))
+# define mpfr_set_exp_t(x,e,r) mpfr_set_si((x),(e),(r))
+#elif defined (_MPFR_H_HAVE_INTMAX_T)
+typedef intmax_t mpfr_eexp_t;
+# define mpfr_get_exp_t(x,r) mpfr_get_sj((x),(r))
+# define mpfr_set_exp_t(x,e,r) mpfr_set_sj((x),(e),(r))
+#else
+# error "Cannot define mpfr_get_exp_t and mpfr_set_exp_t"
+#endif
 
 /* Invalid exponent value (to track bugs...) */
 #define MPFR_EXP_INVALID \
@@ -723,7 +759,7 @@ typedef unsigned long int  mpfr_uexp_t;
 # error "Neither GMP_LIMB_HIGHBIT nor MP_LIMB_T_HIGHBIT defined in GMP"
 #endif
 
-/* Mask to get the Most Significent Bit of a limb */
+/* Mask to get the Most Significant Bit of a limb */
 #define MPFR_LIMB_MSB(l) ((l)&MPFR_LIMB_HIGHBIT)
 
 /* Definition of MPFR_LIMB_ONE & MPFR_LIMB_ZERO*/
@@ -767,7 +803,7 @@ typedef union { mp_size_t s; mp_limb_t l; } mpfr_size_limb_t;
 #else
 #define MPFR_TMP_DECL(x) TMP_DECL
 #define MPFR_TMP_MARK(x) TMP_MARK
-#define MPFR_TMP_ALLOC(s) TMP_SALLOC(s)
+#define MPFR_TMP_ALLOC(s) TMP_ALLOC(s)
 #define MPFR_TMP_FREE(x) TMP_FREE
 #endif
 
@@ -840,6 +876,18 @@ extern unsigned char *mpfr_stack;
 # define MPFR_UNLIKELY(x) (x)
 #endif
 
+/* Declare that some variable is initialized before being used (without a
+   dummy initialization) in order to avoid some compiler warnings. Use the
+   VAR = VAR trick (see http://gcc.gnu.org/bugzilla/show_bug.cgi?id=36296)
+   only with gcc as this is undefined behavior, and we don't know what
+   other compilers do (they may also be smarter). This trick could be
+   disabled with future gcc versions. */
+#if defined(__GNUC__)
+# define INITIALIZED(VAR) VAR = VAR
+#else
+# define INITIALIZED(VAR) VAR
+#endif
+
 /* Ceil log 2: If GCC, uses a GCC extension, otherwise calls a function */
 /* Warning:
  *   Needs to define MPFR_NEED_LONGLONG.
@@ -904,12 +952,12 @@ do {                                                                  \
 #define MPFR_UNSIGNED_MINUS_MODULO(s, a)                              \
   do                                                                  \
     {                                                                 \
-      if ((UINT_MAX % BITS_PER_MP_LIMB) == (BITS_PER_MP_LIMB-1))      \
-        (s) = (mpfr_prec_t) (-(a)) % BITS_PER_MP_LIMB;                \
+      if (IS_POW2 (BITS_PER_MP_LIMB))                                 \
+        (s) = (-(a)) % BITS_PER_MP_LIMB;                              \
       else                                                            \
         {                                                             \
           (s) = (a) % BITS_PER_MP_LIMB;                               \
-          if (s != 0)                                                 \
+          if ((s) != 0)                                               \
             (s) = BITS_PER_MP_LIMB - (s);                             \
         }                                                             \
       MPFR_ASSERTD ((s) >= 0 && (s) < BITS_PER_MP_LIMB);              \
@@ -948,7 +996,34 @@ do {                                                                  \
   } while (0)
 
 /* Needs <locale.h> */
-#define MPFR_DECIMAL_POINT ((unsigned char) localeconv()->decimal_point[0])
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+/* Warning! In case of signed char, the value of MPFR_DECIMAL_POINT may
+   be negative (the ISO C99 does not seem to forbid negative values). */
+#define MPFR_DECIMAL_POINT (localeconv()->decimal_point[0])
+#define MPFR_THOUSANDS_SEPARATOR (localeconv()->thousands_sep[0])
+#else
+#define MPFR_DECIMAL_POINT ((char) '.')
+#define MPFR_THOUSANDS_SEPARATOR ('\0')
+#endif
+
+
+/* Set y to s*significand(x)*2^e, for example MPFR_ALIAS(y,x,1,MPFR_EXP(x))
+   sets y to |x|, and MPFR_ALIAS(y,x,MPFR_SIGN(x),0) sets y to x*2^f such
+   that 1/2 <= |y| < 1. Does not check y is in the valid exponent range.
+   WARNING! x and y share the same mantissa. So, some operations are
+   not valid if x has been provided via an argument, e.g., trying to
+   modify the mantissa of y, even temporarily, or calling mpfr_clear on y.
+*/
+#define MPFR_ALIAS(y,x,s,e)                     \
+  do                                            \
+    {                                           \
+      MPFR_PREC(y) = MPFR_PREC(x);              \
+      MPFR_SIGN(y) = (s);                       \
+      MPFR_EXP(y) = (e);                        \
+      MPFR_MANT(y) = MPFR_MANT(x);              \
+    } while (0)
+
 
 /******************************************************
  **************  Save exponent macros  ****************
@@ -1004,93 +1079,97 @@ typedef struct {
 #define MPFR_RNDRAW_GEN(inexact, dest, srcp, sprec, rnd, sign,              \
                         MIDDLE_HANDLER, OVERFLOW_HANDLER)                   \
   do {                                                                      \
-    mp_size_t dests, srcs;                                                  \
-    mp_limb_t *destp;                                                       \
-    mp_prec_t destprec, srcprec;                                            \
+    mp_size_t _dests, _srcs;                                                \
+    mp_limb_t *_destp;                                                      \
+    mp_prec_t _destprec, _srcprec;                                          \
                                                                             \
     /* Check Trivial Case when Dest Mantissa has more bits than source */   \
-    srcprec = sprec;                                                        \
-    destprec = MPFR_PREC (dest);                                            \
-    destp = MPFR_MANT (dest);                                               \
-    if (MPFR_UNLIKELY (destprec >= srcprec))                                \
+    _srcprec = sprec;                                                       \
+    _destprec = MPFR_PREC (dest);                                           \
+    _destp = MPFR_MANT (dest);                                              \
+    if (MPFR_UNLIKELY (_destprec >= _srcprec))                              \
       {                                                                     \
-        srcs  = (srcprec  + BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB;           \
-        dests = (destprec + BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB - srcs;    \
-        MPN_COPY (destp + dests, srcp, srcs);                               \
-        MPN_ZERO (destp, dests);                                            \
+        _srcs  = (_srcprec  + BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB;         \
+        _dests = (_destprec + BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB - _srcs; \
+        MPN_COPY (_destp + _dests, srcp, _srcs);                            \
+        MPN_ZERO (_destp, _dests);                                          \
         inexact = 0;                                                        \
       }                                                                     \
     else                                                                    \
       {                                                                     \
         /* Non trivial case: rounding needed */                             \
-        mp_prec_t sh;                                                       \
-        mp_limb_t *sp;                                                      \
-        mp_limb_t rb, sb, ulp;                                              \
+        mp_prec_t _sh;                                                      \
+        mp_limb_t *_sp;                                                     \
+        mp_limb_t _rb, _sb, _ulp;                                           \
                                                                             \
         /* Compute Position and shift */                                    \
-        srcs  = (srcprec  + BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB;           \
-        dests = (destprec + BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB;           \
-        MPFR_UNSIGNED_MINUS_MODULO (sh, destprec);                          \
-        sp = srcp + srcs - dests;                                           \
+        _srcs  = (_srcprec  + BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB;         \
+        _dests = (_destprec + BITS_PER_MP_LIMB-1)/BITS_PER_MP_LIMB;         \
+        MPFR_UNSIGNED_MINUS_MODULO (_sh, _destprec);                        \
+        _sp = srcp + _srcs - _dests;                                        \
                                                                             \
         /* General case when prec % BITS_PER_MP_LIMB != 0 */                \
-        if (MPFR_LIKELY (sh != 0))                                          \
+        if (MPFR_LIKELY (_sh != 0))                                         \
           {                                                                 \
-            mp_limb_t mask;                                                 \
+            mp_limb_t _mask;                                                \
             /* Compute Rounding Bit and Sticky Bit */                       \
-            mask = MPFR_LIMB_ONE << (sh-1);                                 \
-            rb = sp[0] & mask;                                              \
-            sb = sp[0] & (mask-1);                                          \
-            if (MPFR_UNLIKELY (sb == 0))                                    \
+            _mask = MPFR_LIMB_ONE << (_sh - 1);                             \
+            _rb = _sp[0] & _mask;                                           \
+            _sb = _sp[0] & (_mask - 1);                                     \
+            if (MPFR_UNLIKELY (_sb == 0))                                   \
               { /* TODO: Improve it */                                      \
-                mp_limb_t *tmp;                                             \
-                mp_size_t n;                                                \
-                for (tmp = sp, n = srcs - dests ; n != 0 && sb == 0 ; n--)  \
-                  sb = *--tmp;                                              \
+                mp_limb_t *_tmp;                                            \
+                mp_size_t _n;                                               \
+                for (_tmp = _sp, _n = _srcs - _dests ;                      \
+                     _n != 0 && _sb == 0 ; _n--)                            \
+                  _sb = *--_tmp;                                            \
               }                                                             \
-            ulp = 2*mask;                                                   \
+            _ulp = 2 * _mask;                                               \
           }                                                                 \
-        else /* sh == 0 */                                                  \
+        else /* _sh == 0 */                                                 \
           {                                                                 \
-            MPFR_ASSERTD (dests < srcs);                                    \
+            MPFR_ASSERTD (_dests < _srcs);                                  \
             /* Compute Rounding Bit and Sticky Bit */                       \
-            rb = sp[-1] & MPFR_LIMB_HIGHBIT;                                \
-            sb = sp[-1] & (MPFR_LIMB_HIGHBIT-1);                            \
-            if (MPFR_UNLIKELY (sb == 0))                                    \
+            _rb = _sp[-1] & MPFR_LIMB_HIGHBIT;                              \
+            _sb = _sp[-1] & (MPFR_LIMB_HIGHBIT-1);                          \
+            if (MPFR_UNLIKELY (_sb == 0))                                   \
               {                                                             \
-                mp_limb_t *tmp;                                             \
-                mp_size_t n;                                                \
-                for (tmp = sp-1, n = srcs - dests-1 ; n!=0 && sb==0 ; n--)  \
-                  sb = *--tmp;                                              \
+                mp_limb_t *_tmp;                                            \
+                mp_size_t _n;                                               \
+                for (_tmp = _sp - 1, _n = _srcs - _dests - 1 ;              \
+                     _n != 0 && _sb == 0 ; _n--)                            \
+                  _sb = *--_tmp;                                            \
               }                                                             \
-            ulp = MPFR_LIMB_ONE;                                            \
+            _ulp = MPFR_LIMB_ONE;                                           \
           }                                                                 \
         /* Rounding */                                                      \
         if (MPFR_LIKELY (rnd == GMP_RNDN))                                  \
           {                                                                 \
-            if (rb == 0)                                                    \
+            if (_rb == 0)                                                   \
               {                                                             \
               trunc:                                                        \
-                inexact = MPFR_LIKELY ((sb | rb) != 0) ? -sign : 0;         \
+                inexact = MPFR_LIKELY ((_sb | _rb) != 0) ? -sign : 0;       \
               trunc_doit:                                                   \
-                MPN_COPY (destp, sp, dests);                                \
-                destp[0] &= ~(ulp-1);                                       \
+                MPN_COPY (_destp, _sp, _dests);                             \
+                _destp[0] &= ~(_ulp - 1);                                   \
               }                                                             \
-            else if (MPFR_UNLIKELY (sb == 0))                               \
+            else if (MPFR_UNLIKELY (_sb == 0))                              \
               { /* Middle of two consecutive representable numbers */       \
                 MIDDLE_HANDLER;                                             \
               }                                                             \
             else                                                            \
               {                                                             \
+                if (0)                                                      \
+                  goto addoneulp_doit; /* dummy code to avoid warning */    \
               addoneulp:                                                    \
                 inexact = sign;                                             \
               addoneulp_doit:                                               \
-                if (MPFR_UNLIKELY (mpn_add_1 (destp, sp, dests, ulp)))      \
+                if (MPFR_UNLIKELY (mpn_add_1 (_destp, _sp, _dests, _ulp)))  \
                   {                                                         \
-                    destp[dests-1] = MPFR_LIMB_HIGHBIT;                     \
+                    _destp[_dests - 1] = MPFR_LIMB_HIGHBIT;                 \
                     OVERFLOW_HANDLER;                                       \
                   }                                                         \
-                destp[0] &= ~(ulp-1);                                       \
+                _destp[0] &= ~(_ulp - 1);                                   \
               }                                                             \
           }                                                                 \
         else                                                                \
@@ -1098,7 +1177,7 @@ typedef struct {
             if (MPFR_LIKELY (MPFR_IS_LIKE_RNDZ (rnd,                        \
                                                 MPFR_IS_NEG_SIGN (sign))))  \
               goto trunc;                                                   \
-             else if (MPFR_UNLIKELY ((sb | rb) == 0))                       \
+             else if (MPFR_UNLIKELY ((_sb | _rb) == 0))                     \
                {                                                            \
                  inexact = 0;                                               \
                  goto trunc_doit;                                           \
@@ -1116,7 +1195,7 @@ typedef struct {
  */
 #define MPFR_RNDRAW(inexact, dest, srcp, sprec, rnd, sign, OVERFLOW_HANDLER) \
    MPFR_RNDRAW_GEN (inexact, dest, srcp, sprec, rnd, sign,                   \
-     if ((sp[0] & ulp) == 0)                                                 \
+     if ((_sp[0] & _ulp) == 0)                                               \
        {                                                                     \
          inexact = -sign;                                                    \
          goto trunc_doit;                                                    \
@@ -1134,7 +1213,7 @@ typedef struct {
 #define MPFR_RNDRAW_EVEN(inexact, dest, srcp, sprec, rnd, sign, \
                          OVERFLOW_HANDLER)                      \
    MPFR_RNDRAW_GEN (inexact, dest, srcp, sprec, rnd, sign,      \
-     if ((sp[0] & ulp) == 0)                                    \
+     if ((_sp[0] & _ulp) == 0)                                  \
        {                                                        \
          inexact = -MPFR_EVEN_INEX * sign;                      \
          goto trunc_doit;                                       \
@@ -1147,14 +1226,15 @@ typedef struct {
      , OVERFLOW_HANDLER)
 
 /* Return TRUE if b is non singular and we can round it to precision 'prec'
-   with rounding mode 'rnd', and with error at most 'error' */
+   and determine the ternary value, with rounding mode 'rnd', and with
+   error at most 'error' */
 #define MPFR_CAN_ROUND(b,err,prec,rnd)                                       \
  (!MPFR_IS_SINGULAR (b) && mpfr_round_p (MPFR_MANT (b), MPFR_LIMB_SIZE (b),  \
                                          (err), (prec) + ((rnd)==GMP_RNDN)))
 
 /* TODO: fix this description (see round_near_x.c). */
 /* Assuming that the function has a Taylor expansion which looks like:
-    y=o(f(x)) = o(x + g(x)) with |g(x)| <= 2^(EXP(v)-err)
+    y=o(f(x)) = o(v + g(x)) with |g(x)| <= 2^(EXP(v)-err)
    we can quickly set y to v if x is small (ie err > prec(y)+1) in most
    cases. It assumes that f(x) is not representable exactly as a FP number.
    v must not be a singular value (NAN, INF or ZERO); usual values are
@@ -1239,8 +1319,8 @@ typedef struct {
    obtain a better error message. The real test should have been a test
    concerning nested functions in gcc, which are disabled by default on
    Darwin; but it is not possible to do that without a configure test. */
-# if !(__MPFR_GNUC(3,0) && __MPFR_GLIBC(2,0))
-#  error "Logging not supported (needs GCC >= 3.0 and GNU C Library >= 2.0)."
+# if defined (__cplusplus) || !(__MPFR_GNUC(3,0) && __MPFR_GLIBC(2,0))
+#  error "Logging not supported (needs gcc >= 3.0 and GNU C Library >= 2.0)."
 # endif
 
 /* Use LOGGING */
@@ -1319,7 +1399,7 @@ __MPFR_DECLSPEC extern mp_prec_t mpfr_log_prec;
 
 #define MPFR_LOG_MSG2(format, ...)                                       \
  if ((MPFR_LOG_MSG_F&mpfr_log_type)&&(mpfr_log_current<=mpfr_log_level)) \
-  fprintf (mpfr_log_file, "%s.%d:"format, __func__, __LINE__, __VA_ARGS__);
+  fprintf (mpfr_log_file, "%s.%d: "format, __func__, __LINE__, __VA_ARGS__);
 #define MPFR_LOG_MSG(x) MPFR_LOG_MSG2 x
 
 #define MPFR_LOG_BEGIN2(format, ...)                                         \
@@ -1511,6 +1591,9 @@ __MPFR_DECLSPEC int mpfr_exp_2 _MPFR_PROTO ((mpfr_ptr, mpfr_srcptr,mp_rnd_t));
 __MPFR_DECLSPEC int mpfr_exp_3 _MPFR_PROTO ((mpfr_ptr, mpfr_srcptr,mp_rnd_t));
 __MPFR_DECLSPEC int mpfr_powerof2_raw _MPFR_PROTO ((mpfr_srcptr));
 
+__MPFR_DECLSPEC int mpfr_pow_general _MPFR_PROTO ((mpfr_ptr, mpfr_srcptr,
+                           mpfr_srcptr, mp_rnd_t, int, mpfr_save_expo_t *));
+
 __MPFR_DECLSPEC void mpfr_setmax _MPFR_PROTO ((mpfr_ptr, mp_exp_t));
 __MPFR_DECLSPEC void mpfr_setmin _MPFR_PROTO ((mpfr_ptr, mp_exp_t));
 
