@@ -1,22 +1,22 @@
 /* Test file for exceptions.
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -65,6 +65,11 @@ check_default_rnd (void)
 static void
 check_emin_emax (void)
 {
+  mp_exp_t old_emin, old_emax;
+
+  old_emin = mpfr_get_emin ();
+  old_emax = mpfr_get_emax ();
+
   /* Check the functions not the macros ! */
   if ((mpfr_set_emin)(MPFR_EMIN_MIN) != 0)
     ERROR("set_emin failed!");
@@ -88,6 +93,9 @@ check_emin_emax (void)
     ERROR ("get_emax_min");
   if ((mpfr_get_emax_max) () != MPFR_EMAX_MAX)
     ERROR ("get_emax_max");
+
+  set_emin (old_emin);
+  set_emax (old_emax);
 }
 
 static void
@@ -131,7 +139,11 @@ static void
 check_flags (void)
 {
   mpfr_t x;
-  mpfr_init(x);
+  mp_exp_t old_emin, old_emax;
+
+  old_emin = mpfr_get_emin ();
+  old_emax = mpfr_get_emax ();
+  mpfr_init (x);
 
   /* Check the functions not the macros ! */
   (mpfr_clear_flags)();
@@ -168,7 +180,9 @@ check_flags (void)
   if (!(mpfr_erangeflag_p)())
     ERROR ("ERROR: No erange flag!\n");
 
-  mpfr_clear(x);
+  mpfr_clear (x);
+  set_emin (old_emin);
+  set_emax (old_emax);
 }
 
 static void
@@ -258,7 +272,8 @@ test_set_overflow (void)
 }
 
 static void
-check_set () {
+check_set (void)
+{
   mpfr_clear_flags ();
 
   mpfr_set_overflow ();
@@ -368,6 +383,9 @@ main (int argc, char *argv[])
   mpfr_clear (x);
   mpfr_clear (y);
 
+  set_emin (emin);
+  set_emax (emax);
+
   check_emin_emax();
   check_flags();
   check_set_get_prec ();
