@@ -1,22 +1,23 @@
 /* Test file for mpfr_set_exp.
 
-Copyright 2004 Free Software Foundation.
+Copyright 2004, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin Place, Fifth Floor, Boston,
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
+the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
 #include <stdio.h>
@@ -29,9 +30,13 @@ main (int argc, char *argv[])
 {
   mpfr_t x;
   int ret;
+  mp_exp_t emin, emax;
 
   tests_start_mpfr ();
 
+  emin = mpfr_get_emin ();
+  emax = mpfr_get_emax ();
+
   mpfr_init (x);
 
   mpfr_set_ui (x, 1, GMP_RNDN);
@@ -54,6 +59,9 @@ main (int argc, char *argv[])
 
   mpfr_clear (x);
 
+  set_emin (emin);
+  set_emax (emax);
+
   tests_end_mpfr ();
   return 0;
 }
