@@ -1,22 +1,22 @@
 /* Test file for mpfr_set_str.
 
-Copyright 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+Copyright 2004, 2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 Contributed by the Arenaire and Cacao projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
+along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301, USA. */
 
@@ -24,32 +24,10 @@ MA 02110-1301, USA. */
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
-#include <time.h>
 
 #include "mpfr-test.h"
 
-static void check_reftable (void);
-static void check_special  (void);
-static void check_retval   (void);
-static void check_overflow (void);
-static void check_parse    (void);
-
-int
-main (int argc, char *argv[])
-{
-  tests_start_mpfr ();
-
-  check_special();
-  check_reftable ();
-  check_parse ();
-  check_overflow ();
-  check_retval ();
-
-  tests_end_mpfr ();
-  return 0;
-}
-
-void
+static void
 check_special (void)
 {
   mpfr_t x, y;
@@ -73,36 +51,56 @@ check_special (void)
     }
 
   /* Check NAN  */
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
   res = mpfr_strtofr (x, "NaN", &s, 10, GMP_RNDN);
   if (res != 0 || !mpfr_nan_p (x) || *s != 0)
     {
       printf ("Error for setting NAN (1)\n");
       exit (1);
     }
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
+  res = mpfr_strtofr (x, "+NaN", &s, 10, GMP_RNDN);
+  if (res != 0 || !mpfr_nan_p (x) || *s != 0)
+    {
+      printf ("Error for setting +NAN (1)\n");
+      exit (1);
+    }
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
+  res = mpfr_strtofr (x, " -NaN", &s, 10, GMP_RNDN);
+  if (res != 0 || !mpfr_nan_p (x) || *s != 0)
+    {
+      printf ("Error for setting -NAN (1)\n");
+      exit (1);
+    }
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
   res = mpfr_strtofr (x, "@nAn@xx", &s, 16, GMP_RNDN);
   if (res != 0 || !mpfr_nan_p (x) || strcmp(s, "xx") )
     {
       printf ("Error for setting NAN (2)\n");
       exit (1);
     }
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
   res = mpfr_strtofr (x, "NAN(abcdEDF__1256)Hello", &s, 10, GMP_RNDN);
   if (res != 0 || !mpfr_nan_p (x) || strcmp(s, "Hello") )
     {
       printf ("Error for setting NAN (3)\n");
       exit (1);
     }
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
   res = mpfr_strtofr (x, "NAN(abcdEDF)__1256)Hello", &s, 10, GMP_RNDN);
   if (res != 0 || !mpfr_nan_p (x) || strcmp(s, "__1256)Hello") )
     {
       printf ("Error for setting NAN (4)\n");
       exit (1);
     }
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
   res = mpfr_strtofr (x, "NAN(abc%dEDF)__1256)Hello", &s, 10, GMP_RNDN);
   if (res != 0 || !mpfr_nan_p (x) || strcmp(s, "(abc%dEDF)__1256)Hello") )
     {
       printf ("Error for setting NAN (5)\n");
       exit (1);
     }
+  mpfr_set_ui (x, 0, GMP_RNDN); /* make sure that x is modified */
   res = mpfr_strtofr (x, "NAN((abc))", &s, 10, GMP_RNDN);
   if (res != 0 || !mpfr_nan_p (x) || strcmp(s, "((abc))") )
     {
@@ -191,7 +189,8 @@ int randomab (int a, int b)
   return a + rand () % (b-a);
 }
 
-int main()
+int
+main (void)
 {
   int i, base;
   mpfr_t x;
@@ -211,7 +210,7 @@ int main()
       base = randomab (2, 30);
       e = randomab (-1<<15, 1<<15);
       mpfr_set_prec (x, p);
-      mpfr_random (x);
+      mpfr_urandomb (x, RANDS);
       mpfr_mul_2si (x, x, e, GMP_RNDN);
       printf("{%lu, %d,\n\"", p, base);
       mpfr_out_str (stdout, base, p, x, GMP_RNDN);
@@ -532,9 +531,8 @@ static struct dymmy_test {
 "1.001000010110011011000101100000101111101001100011101101001111110111000010010110010001100e-16920"}
 };
 
-
-void
-check_reftable ()
+static void
+check_reftable (void)
 {
   int i, base;
   mpfr_t x, y;
@@ -578,14 +576,34 @@ check_reftable ()
   mpfr_clear (x);
 }
 
-void
+static void
 check_parse (void)
 {
   mpfr_t x;
   char *s;
+  int res;
 
   mpfr_init (x);
 
+  /* Invalid data */
+  mpfr_set_si (x, -1, GMP_RNDN);
+  res = mpfr_strtofr (x, "  invalid", NULL, 10, GMP_RNDN);
+  if (MPFR_NOTZERO (x) || MPFR_IS_NEG (x))
+    {
+      printf ("Failed parsing '  invalid' (1)\n X=");
+      mpfr_dump (x);
+      exit (1);
+    }
+  MPFR_ASSERTN (res == 0);
+  mpfr_set_si (x, -1, GMP_RNDN);
+  res = mpfr_strtofr (x, "  invalid", &s, 0, GMP_RNDN);
+  if (MPFR_NOTZERO (x) || MPFR_IS_NEG (x) || strcmp (s, "  invalid"))
+    {
+      printf ("Failed parsing '  invalid' (2)\n S=%s\n X=", s);
+      mpfr_dump (x);
+      exit (1);
+    }
+  MPFR_ASSERTN (res == 0);
   /* Check if it stops correctly */
   mpfr_strtofr (x, "15*x", &s, 10, GMP_RNDN);
   if (mpfr_cmp_ui (x, 15) || strcmp (s, "*x"))
@@ -617,6 +635,63 @@ check_parse (void)
       mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
       exit (1);
     }
+  /* P form */
+  mpfr_strtofr (x, "0x42P17", &s, 16, GMP_RNDN);
+  if (mpfr_cmp_str (x, "8650752", 10, GMP_RNDN) || *s != 0)
+    {
+      printf ("Failed parsing '0x42P17' (base = 16)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
+  mpfr_strtofr (x, "-0X42p17", &s, 16, GMP_RNDN);
+  if (mpfr_cmp_str (x, "-8650752", 10, GMP_RNDN) || *s != 0)
+    {
+      printf ("Failed parsing '-0x42p17' (base = 16)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
+  mpfr_strtofr (x, "42p17", &s, 16, GMP_RNDN);
+  if (mpfr_cmp_str (x, "8650752", 10, GMP_RNDN) || *s != 0)
+    {
+      printf ("Failed parsing '42p17' (base = 16)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
+  mpfr_strtofr (x, "-42P17", &s, 16, GMP_RNDN);
+  if (mpfr_cmp_str (x, "-8650752", 10, GMP_RNDN) || *s != 0)
+    {
+      printf ("Failed parsing '-42P17' (base = 16)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
+  mpfr_strtofr (x, "0b1001P17", &s, 2, GMP_RNDN);
+  if (mpfr_cmp_str (x, "1179648", 10, GMP_RNDN) || *s != 0)
+    {
+      printf ("Failed parsing '0b1001P17' (base = 2)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
+  mpfr_strtofr (x, "-0B1001p17", &s, 2, GMP_RNDN);
+  if (mpfr_cmp_str (x, "-1179648", 10, GMP_RNDN) || *s != 0)
+    {
+      printf ("Failed parsing '-0B1001p17' (base = 2)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
+  mpfr_strtofr (x, "1001p17", &s, 2, GMP_RNDN);
+  if (mpfr_cmp_str (x, "1179648", 10, GMP_RNDN) || *s != 0)
+    {
+      printf ("Failed parsing '1001p17' (base = 2)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
+  mpfr_strtofr (x, "-1001P17", &s, 2, GMP_RNDN);
+  if (mpfr_cmp_str (x, "-1179648", 10, GMP_RNDN) || *s != 0)
+    {
+      printf ("Failed parsing '-1001P17' (base = 2)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
   /* Check for auto-detection of the base */
   mpfr_strtofr (x, "+0x42P17", &s, 0, GMP_RNDN);
   if (mpfr_cmp_str (x, "42P17", 16, GMP_RNDN) || *s != 0)
@@ -632,6 +707,13 @@ check_parse (void)
       mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
       exit (1);
     }
+  mpfr_strtofr (x, "-42P17", &s, 0, GMP_RNDN);
+  if (mpfr_cmp_str (x, "-42", 10, GMP_RNDN) || strcmp (s, "P17"))
+    {
+      printf ("Failed parsing '-42P17' (base = 0)\n S='%s'\n X=", s);
+      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
+      exit (1);
+    }
   mpfr_strtofr (x, " 0b0101.011@42", &s, 0, GMP_RNDN);
   if (mpfr_cmp_str (x, "0101.011@42", 2, GMP_RNDN) || *s != 0)
     {
@@ -702,7 +784,7 @@ check_parse (void)
       mpfr_out_str (stdout, 10, 0, x, GMP_RNDN); putchar ('\n');
       exit (1);
     }
-  /* Check for 0x */
+  /* Check for 0x and 0b */
   mpfr_strtofr (x, "  0xG", &s, 0, GMP_RNDN);
   if (mpfr_cmp_ui (x, 0) || strcmp(s,"xG"))
     {
@@ -848,3 +930,104 @@ check_retval (void)
 
   mpfr_clear (x);
 }
+
+/* Bug found by Christoph Lauter (in mpfr_set_str). */
+static struct bug20081025_test {
+  mpfr_rnd_t rnd;
+  int inexact;
+  const char *str;
+  const char *binstr;
+} Bug20081028Table[] = {
+  {GMP_RNDN, -1, "1.00000000000000000006", "1"},
+  {GMP_RNDZ, -1, "1.00000000000000000006", "1"},
+  {GMP_RNDU, +1, "1.00000000000000000006",
+   "10000000000000000000000000000001e-31"},
+  {GMP_RNDD, -1, "1.00000000000000000006", "1"},
+
+
+  {GMP_RNDN, +1, "-1.00000000000000000006", "-1"},
+  {GMP_RNDZ, +1, "-1.00000000000000000006", "-1"},
+  {GMP_RNDU, +1, "-1.00000000000000000006", "-1"},
+  {GMP_RNDD, -1, "-1.00000000000000000006",
+   "-10000000000000000000000000000001e-31"},
+
+  {GMP_RNDN, +1, "0.999999999999999999999999999999999999999999999", "1"},
+  {GMP_RNDZ, -1, "0.999999999999999999999999999999999999999999999",
+   "11111111111111111111111111111111e-32"},
+  {GMP_RNDU, +1, "0.999999999999999999999999999999999999999999999", "1"},
+  {GMP_RNDD, -1, "0.999999999999999999999999999999999999999999999",
+   "11111111111111111111111111111111e-32"},
+
+  {GMP_RNDN, -1, "-0.999999999999999999999999999999999999999999999", "-1"},
+  {GMP_RNDZ, +1, "-0.999999999999999999999999999999999999999999999",
+   "-11111111111111111111111111111111e-32"},
+  {GMP_RNDU, +1, "-0.999999999999999999999999999999999999999999999",
+   "-11111111111111111111111111111111e-32"},
+  {GMP_RNDD, -1, "-0.999999999999999999999999999999999999999999999", "-1"}
+};
+
+static void
+bug20081028 (void)
+{
+  int i;
+  int inexact, res;
+  mpfr_rnd_t rnd;
+  mpfr_t x, y;
+  char *s;
+
+  mpfr_init2 (x, 32);
+  mpfr_init2 (y, 32);
+  for (i = 0 ; i < numberof (Bug20081028Table) ; i++)
+    {
+      rnd     = Bug20081028Table[i].rnd;
+      inexact = Bug20081028Table[i].inexact;
+      mpfr_set_str_binary (x, Bug20081028Table[i].binstr);
+      res = mpfr_strtofr (y, Bug20081028Table[i].str, &s, 10, rnd);
+      if (s == NULL || *s != 0)
+        {
+          printf ("Error in Bug20081028: strtofr didn't parse entire input\n"
+                  "for (i=%d) Str=\"%s\"", i, Bug20081028Table[i].str);
+          exit (1);
+        }
+      if (! SAME_SIGN (res, inexact))
+        {
+          printf ("Error in Bug20081028: expected %s ternary value, "
+                  "got %d\nfor (i=%d) Rnd=%s Str=\"%s\"\n Set binary gives: ",
+                  inexact > 0 ? "positive" : "negative",
+                  res, i, mpfr_print_rnd_mode(rnd), Bug20081028Table[i].str);
+          mpfr_dump (x);
+          printf (" strtofr    gives: ");
+          mpfr_dump (y);
+          exit (1);
+        }
+      if (mpfr_cmp (x, y))
+        {
+          printf ("Error in Bug20081028: Results differ between strtofr and "
+                  "set_binary\nfor (i=%d) Rnd=%s Str=\"%s\"\n"
+                  " Set binary gives: ",
+                  i, mpfr_print_rnd_mode(rnd), Bug20081028Table[i].str);
+          mpfr_dump (x);
+          printf (" strtofr    gives: ");
+          mpfr_dump (y);
+          exit (1);
+        }
+    }
+  mpfr_clear (y);
+  mpfr_clear (x);
+}
+
+int
+main (int argc, char *argv[])
+{
+  tests_start_mpfr ();
+
+  check_special();
+  check_reftable ();
+  check_parse ();
+  check_overflow ();
+  check_retval ();
+  bug20081028 ();
+
+  tests_end_mpfr ();
+  return 0;
+}
